# Metrics.pm (HCT::Metrics).
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

package HCT::Metrics;

use HCT::Metrics::McCabe;

# Metrics.pm (HCT::Metrics::Module)

package HCT::Metrics::Module;

sub new
{
	my $inv = shift;
	my $class = ref($inv) || $inv;
	my $self = {
		module => undef,
		current => undef,
	};
	bless $self, $class;
	return $self;
}

#---------------------------------------------------------------------------
# Method:      module
# Description: get current module
# Returns:     module
#---------------------------------------------------------------------------

sub current
{
	$_[0]->{current};
}

#---------------------------------------------------------------------------
# Method:      metrics
# Description: metrics of current module
# Returns:     list
#---------------------------------------------------------------------------

sub metrics
{
	$_[0]->current->{metrics};
}

#---------------------------------------------------------------------------
# Method:      new_module
# Description: structure of new module
# Returns:     module
#---------------------------------------------------------------------------

sub new_module ($)
{
	my ($self, $name) = (shift, shift);
	return {
		name	=> $name, #
		prev	=> undef, #      prev <- name -> next
		next	=> undef, #                |
		inner	=> undef, #              inner
		metrics	=> { },
	};
}

#---------------------------------------------------------------------------
# Method:      create_module
# Description: creates a new module
# Returns:     
#---------------------------------------------------------------------------

sub create_module
{
	my ($self, $name) = (shift, shift);

	# if module is undefined 
	if (!defined $self->{module}) {
		$self->{module} = $self->{current} = $self->new_module($name);
	}
	# else just add a new one
	else {
		$self->add_module ($name, $self->last_module);
	}
}

#---------------------------------------------------------------------------
# Method:      create_innermodule
# Description: creates an innermodule 
# Returns:     
#---------------------------------------------------------------------------

sub create_innermodule
{
	my ($self, $name) = (shift, shift);

	# If this module will be first
	if (!defined $self->{current}->{inner}) {
		my $save_module = $self->{current};
		$self->{current} = $self->{current}->{inner} = $self->new_module ($name);
		$self->{current}->{prev} = $save_module; 
	}

	# Else just add it to the end of already exists
	else {
		$self->add_module( $name, $self->last_module ($self->{current}->{inner}) );
	}	 
}

#---------------------------------------------------------------------------
# Method:      add_module
# Description: adds a new module and makes connections
# Returns:     
#---------------------------------------------------------------------------

sub add_module
{
	my ($self, $name, $module) = (shift, shift, shift);
	my $save_module = $self->{current};
	$self->{current} = $module->{next} = $self->new_module($name);
	$self->{current}->{prev} = $save_module;
}

#---------------------------------------------------------------------------
# Method:      last_module
# Description: get last module from current list
# Returns:     last module
#---------------------------------------------------------------------------

sub last_module
{
	my $self = shift;
	
	# if current module is the last
	if (!defined $self->{current}->{next}) {
		return $self->{current};
	}
	
	# else find the last module
	my $last_module = $self->{current};
	while ( defined $last_module->{next} ) {
		$last_module = $last_module->{next};
	}
	return $last_module;
}

#---------------------------------------------------------------------------
# Method:      close_module
# Description: closes current modudle and moves to last 
# Returns:     
#---------------------------------------------------------------------------

sub close_module
{
	my $self = shift;
	$self->{current} = $self->{current}->{prev};
}

# Metrics.pm (HCT::Metrics::Obj)

package HCT::Metrics::Obj;

use strict;
use warnings;

sub AUTOLOAD
{
	return HCT::AUTOLOAD(
		(caller),
		our $AUTOLOAD,
		@_
	);
}

#---------------------------------------------------------------------------
# Method:      new
# Description: constructor
# Returns:
#---------------------------------------------------------------------------

sub new
{
	my $inv = shift;
	my $class = ref($inv) || $inv;
	my $self = { };
	bless $self, $class;
	return $self;
}

#---------------------------------------------------------------------------
# Method:      sum
# Description: 
# Returns:
#---------------------------------------------------------------------------

sub sum ($$)
{
	my ($self, $s1, $s2) = (shift, shift, shift);
	return $s1 + $s2;
}

#---------------------------------------------------------------------------
# Method:      compute
# Description:
# Returns:
#---------------------------------------------------------------------------

sub compute ($@)
{
	my ($self, $metricslist) = (shift, shift);
	return ();
}

1;

__END__



