
package HCT::Output;

use strict;
use warnings;

use HCT::Std;
use HCT::Util;
use HCT::Constants;
use HCT::File;

sub AUTOLOAD
{
	return HCT::AUTOLOAD(
		our $AUTOLOAD,  # current AUTOLOAD
		@_
	);
}

use constant WIDTH => 0;
use constant VALUE => 1;

sub new
{
	my $inv = shift;
	my $class = ref($inv) || $inv;
	my $self = {
		metrics => { },
		level => 0,
		levels  => { },
		no_modules => FALSE,	
		init => FALSE,
		max_width => 72, # default max width
		columns => [
			[18, 'FILENAME'],
			[12, 'MODULE'],
		],
		num_columns => 0,
		separator => {
			solid	=> "",
			empty	=> "",
			divided	=> "",
		},
	};
	bless $self, $class;
	return $self;
}

sub metrics
{
	my ($self, $metrics) = (shift, shift);
	
	if (defined $metrics) {
		$self->{metrics} = $metrics;
		return;
	}
	
	$self->{metrics};
}

sub init
{
	my $self = shift;
	
	if ( $self->{init} == TRUE ) {
		return;
	}
	
	my $format = $HCT::OUTPUT_FORMAT;
	
	if ( $HCT::OUTPUT_AUTO ) {

		if ( !defined $self->levels->{$format} ) {
			print STDERR 'Unknown format';
			exit;
		}

		my $level = $self->levels->{$format}->[ $self->{level} ];
		$self->{max_width} = $level->[WIDTH];
		
		foreach my $column ( @{ $level->[VALUE] } ) {
			if ( defined $self->{metrics}->{$column->[VALUE]} ) {
				push @{ $self->{columns} }, $column;
			}
		}

	}
	
	if ( $self->{no_modules} ) {
		splice (@{$self->{columns}},1,1);
	}
	
	my $max_width = 1;
	
	for ( @{ $self->{columns} } ) {
		$max_width += $_->[WIDTH] + 3;
	} # _ + _ + | <== 3

	while (1) {
		if ( $self->{max_width} < $max_width ) {
			my $column = pop @{ $self->{columns} };
			$max_width -= $column->[WIDTH];
		} else {
			last;
		}
	}

	if ( $self->{max_width} > $max_width ) {

		my $d = $self->{max_width} - $max_width;
		WC: while ( 1 ) {
			
				foreach ( @{ $self->{columns} } ) {
					last WC if $d <= 0;
					$_->[WIDTH] += 1;
					$d--;
				}
			
		}
		
	}

	$self->{num_columns} = scalar @{$self->{columns}};

	$self->{separator}->{solid} = $self->{separator}->{divided} = '+';
	foreach my $column (@{$self->{columns}}) {
		$self->{separator}->{divided} .= "-" . ("-" x $column->[WIDTH]) . "-+";
		$self->{separator}->{solid}   .= "-" . ("-" x $column->[WIDTH]) . "--";
	}
	chop $self->{separator}->{solid};
	$self->{separator}->{solid} .= "+\n";
	$self->{separator}->{divided} .= "\n";

	$self->{init} = TRUE;
}

sub output
{
	my ($self, $lang, $filelist) = (shift, shift, shift);
	
	if ( $self->{level} == 0 ) {
		$self->{level} = $HCT::OUTPUT_LEVEL;
	}
	
	$self->init;

	$self->header;

	my $done = scalar keys %$filelist;
	
	while ( my ($filename, $modules) = each %$filelist ) {
		
		my $to = ($self->{no_modules} == FALSE) ? scalar(@$modules) - 1 : 0;
		for (my $i=0; $i <= $to; $i++) {

			$self->put ( $self->create_form (' ') );
			
			my @a;
			
			foreach ( @{ $self->{columns} } ) {
				
				if ( $_->[VALUE] eq 'FILENAME' ) {
					push @a, ($i == 0) ? reduce_string (get_basename ($filename), $_->[0]) : '';					
				}
				elsif ($_->[VALUE] eq 'MODULE') {
					push @a, reduce_string ($modules->[$i]->[0], $_->[0]);
				}
				else {
					push @a, (defined $modules->[$i]->[VALUE]->{$_->[VALUE]}) ? $modules->[$i]->[VALUE]->{$_->[VALUE]} : '';
				} 
			}
			#print join ',' => @a;
			$self->fill(  @a );
		}

		$done--;
		
		if ( $done > 0 ) {
			print $self->{separator}->{solid} if $HCT::OUTPUT_FORMAT eq 'TERMINAL';
			#print "\n" if $HCT::OUTPUT_FORMAT eq 'CSV';
		} 
		
	}

	$self->footer;
}

sub exclude
{
	my $self = shift;
	my $metrics = shift;
	
	foreach (@$metrics) {
		if ( defined $self->{metrics}->{$_} ) {
			delete $self->{metrics}->{$_};
		}
	}
	
}

#sub include
#{
#	my $self = shift;
#	my $metrics = shift;
#
#	foreach (@$metrics) {
#		#$self->{metrics}->{$_}
#	}
#}

sub create_form
{
	my ($self, $separator, $form) = (shift, shift, '');

	$separator = ',' if $HCT::OUTPUT_FORMAT eq 'CSV';

	$form .= "|" if $HCT::OUTPUT_FORMAT eq 'TERMINAL';
	$form .= "" if $HCT::OUTPUT_FORMAT eq 'CSV';
	
	$form .= join $separator =>
		map {
			' @' . ('<' x ($_->[WIDTH]-1)) . ' '
		} @{ $self->{columns} };
	$form .= "|" if $HCT::OUTPUT_FORMAT eq 'TERMINAL';
	$form .= "" if $HCT::OUTPUT_FORMAT eq 'CSV';
	$form .= "\n";

	my @vars;
	for (1 .. scalar(@{$self->{columns}})) {
		push @vars, "\$_[" . ($_-1) . "]";
	}
	$form .= join ',' => @vars;
	
	return $form;
}

sub header
{
	my $self = shift;
	my $header = "";
	
	
	print $self->{separator}->{divided} if $HCT::OUTPUT_FORMAT eq 'TERMINAL';
	print "\n" if $HCT::OUTPUT_FORMAT eq 'CSV';
	
	$self->put  ( $self->create_form ('|') );
	$self->fill ( map { $_->[VALUE] } @{ $self->{columns} } );
	
	print $self->{separator}->{divided} if $HCT::OUTPUT_FORMAT eq 'TERMINAL';
	#print "\n" if $HCT::OUTPUT_FORMAT eq 'CSV';
}

sub form
{
	my $self = shift;
	my ($start,$sep, $columns, $end) = (@_);
	my $form = "";

	$form .= $start;
	$form .= join $sep => map { '@' . ('|' x $_->[WIDTH]) . " " } @$columns;
	$form .= $end . "\n";
	
	my @vars = ();
	for (1 .. scalar(@$columns)) {
		push @vars, "\$_[" . ($_-1) . "]";
	}
	$form .= join ',' => @vars;
	$form .= "\n";

	$self->put($form);
	$self->fill();
}

sub footer
{
	my $self = shift;
	print $self->{separator}->{solid} if $HCT::OUTPUT_FORMAT eq 'TERMINAL';
	print "\n" if $HCT::OUTPUT_FORMAT eq 'CSV';
}

sub clear
{
	$^A = ''
}

sub fill
{
	shift;
	write ();
}

sub put
{
	my ($self,$format) = (shift,shift);
no warnings;
	$^A = '';

	eval "format STDOUT = \n" . $format . "\n.\n";
	
	if ($@) {
		print "format STDOUT = \n" . $format . "\n.\n";		
		print $@;
		exit(0);
	}
}

sub levels
{
	my $self = shift;
	
	if (defined $_[0]) {
		$self->{levels} = $_[0];
		return;
	}
	
	$self->{levels};
}

sub print_levels
{
	my $self = shift;
	
	while ( my ($format,$levels) = each %{$self->{levels}} ) {
		print "Format $format:\n";
		
		for (my $i=1; $i<scalar(@$levels); $i++) {
			my $level = $levels->[$i];
			print " level $i, width $level->[0]: ";
			print "empty" if ( (!defined $level->[1]) || (scalar($level->[1]) == 0) );
			print join ", " => map { $_->[1] } @{ $level->[1] };
			print ".\n"			
		}
		print "\n";
	}
	exit;
}

sub print_metrics
{
	my $self = shift;
	print "Avaliable metrics: ";
	print join ', ' => keys %{$self->{metrics}};
	print ".\n\n";
}

sub DESTROY
{
	
}

1;

